/* COPYRIGHT (C) GUPTA TECHNOLOGIES, INC. 1984, 1985, 1986, 1987 */
/*
FUNCTION NAME
  pbgapi - C Program Example to test server api
REVISION HISTORY
  08/15/88 DWN original
  11/12/90 CHY modified to use servernames instead of server number.
	       removed the use of hdrmgc field.
EXTERNAL INTERFACE
  FUNCTION
    Main function of the server api test program.
  DESCRIPTION
    The is a sample program to demonstrate use of the server api functions
    sqlcsv(), sqlgsi(), and sqldsv().
  RETURN
INTERNAL DESCRIPTION
  Connect to the server, get server information every two seconds until
  a keystroke is hit, and disconnect from server.

  The program is invoked by:
    PBGAPI -s <servername> -p <server password>
  The default servername is null, if not specified.  The default
  password is no password at all.
*/
#define INCL_DOS
#include "stdio.h"
#include "string.h"
#include "os2.h"
#include "sql.h"
#include "sqlsrv.h"

char *Helptext[] =
{
  "%C - Test server API calls for OS/2 SQLBASE",
  "",
  "Copyright Gupta Technologies, Inc 1988",
  "",
  "Usage: %C [options]",
  "",
  "Options:",
  "\ \ -p <password> Server password",
  "\ \ -s <servername> servername ",
  "\ \ -h This help message",
  0
};

/* some simple typedefs */
typedef char b1;
typedef unsigned char ub1;
typedef short int b2;
typedef unsigned short int ub2;
typedef long b4;
typedef unsigned long ub4;

#define SSHORT(p) ((b2*)(p))[0]

void help(void);

char *Command;
char Buffer[6000];

main(argc, argv)
int argc;
char **argv;
{
  SQLTSVH handle;
  b2 ret;
  char *p;
  int c;
  char *password;
  SQLTDAP srvname;
  int loop;
  
  p = *argv++; argc--;
  Command = p;
  while (c = *p++)
    if (*p == '/' || *p == '\\' || *p == ':')
      Command = p;
  if ((p = strchr(Command, '.')) != 0)
    *p = '\0';
  
  srvname = "";
  password = 0;

  while (argc-- > 0)
  {
    p = *argv++;
    if (*p == '-')
    {
      p++;
      while (*p)
      {
	switch (*p++)
	{
	case 'h': case 'H':
	  help();
	  break;
	case 'p': case 'P':
	  if (argc-- > 0)
	    password = *argv++;
	  else
	  {
	    doerror("Missing password argument\n");
	    exit(1);
	  }
	  break;
	case 's': case 'S':
	  if (argc-- > 0)
	  {
	    srvname = *argv++;
          }
	  else
	  {
	      printf("Missing server name argument\n");
	      exit(1);
	  }
	  break;
	default:
	  doerror("Unknown option -%c\n", p[-1]);
	}
      }
    }
    else
    {
      doerror("Unknown argument type \"%s\"\n", p);
    }
  }

  printf("servername: %s\n", srvname);
  printf("Password: %s\n",
      (password && *password) ? password : "NO PASSWORD");

  /* connect to server */
  if ((ret = sqlcsv(&handle, srvname, password)) != 0)
    doerror("Cannot connect to server - %u\n", ret);

  /* while there is no keystrokes, perform periodic update */
  for (loop=0; loop < 10 ; loop++)
  {
    /* if (kbhit())
    {
      while (kbhit())
	getch();
      printf("Terminating\n");
      break;
    } */
    if (doit(handle))
    {
      printf("Terminating due to error\n");
      break;
    }
    /* DosSleep(2000L); */
    printf("Terminating \n");
    
  }
  
  /* disconnect from server */
  if ((ret = sqldsv(handle)) != 0)
    doerror("Error disconnecting from server - %u\n", ret);
  printf("Done\n");
  exit(0);
}

doit(handle)
SQLTSVH handle;
{
  SQLTRCD ret;
  SQLTFLG flags;
  SQLTDAL rbuflen;
  char *p;
  hdrdef* hdr;
  mshdef* msh;
  int length;
  char *datap;
  
  flags = SQLGCUR | SQLGDBS | SQLGCFG | SQLGSTT | SQLGPRC;

  if ((ret = sqlgsi(handle, flags, Buffer, sizeof(Buffer), &rbuflen)) != 0)
    doprint("Error getting server information - %u\n", ret);

  p = Buffer;
  if (rbuflen < sizeof(hdrdef))
  {
    doerror("Message too small to contain header\n");
    return ret;
  }

  hdr = (hdrdef*) p;
  p += sizeof(hdrdef);

  length = hdr->hdrlen - sizeof(hdrdef);

  while (length > sizeof(mshdef))
  {
    msh = (mshdef*) p;
    datap = (b1*) (msh + 1);
    p += msh->mshlen;
    length -= msh->mshlen;
    switch (msh->mshflg)
    {
    case SQLGCUR:			/* cursor information */
      rptCur(msh, (curdef*) datap);
      break;
    case SQLGDBS:			/* database information */
      rptDbs(msh, (dbsdef*) datap);
      break;
    case SQLGCFG:			/* configuration information */
      rptCfg(msh, (cfgdef*) datap);
      break;
    case SQLGSTT:			/* statistics */
      rptStt(msh, (sttdef*) datap);
      break;
    case SQLGPRC:			/* process information */
      rptPrc(msh, (prcdef*) datap);
      break;
    default:
      doprint("Unknown message section type - 0x%X\n", msh->mshflg);
      break;
    }
  }

  return ret;
}

/* report cursor information */
rptCur(msh, cur)
mshdef* msh;
curdef *cur;
{
  int i;

  printf("\nCURSORS:\n\n");
  printf("NumRows InLen OutLn PhyRd PhyWr VirRd "
      "VirWr CT P# LI User Name \ \ \ DbName\n");
  for (i = 0; i < msh->mshnen; cur++, i++)
  {
    printf("%7lu %5u %5u %5u %5u %5u %5u %2u %2u %2s %-12s %-9s\n",
        cur->currow,			/* number of rows */
	cur->curibl,			/* input buffer */
	cur->curobl,			/* output message buffer length */
	cur->curspr,			/* stat counter, physical reads */
	cur->curspw,			/* stat counter, physical writes */
	cur->cursvr,			/* stat counter, virtual reads */
	cur->cursvw,			/* stat counter, virtual writes */
	cur->curtyp,			/* command type */
	cur->curpnm,			/* process number */
	cur->curiso,			/* locking isolation */
	cur->curunb,			/* user name buffer */
	cur->curdbn);			/* database name */
  }
}

/* report database information */
rptDbs(msh, dbs)
mshdef* msh;
dbsdef *dbs;
{
  int i;

  printf("\nDATABASES:\n\n");
  for (i = 0; i < msh->mshnen; dbs++, i++)
  {
    printf("Database Name: \"%s\"\n",  dbs->dbsfnm);
    printf("Before Image Size: %lu\n", dbs->dbsbfs);
    printf("Before image write page: %lu\n", dbs->dbsbwp);
    printf("Database file size: %lu\n", dbs->dbsdfs);
    printf("1st reader page in circular log: %lu\n", dbs->dbsfrp);
    printf("1st updater page in circular log: %lu\n", dbs->dbsfup);
    printf("last page allocated: %lu\n", dbs->dbslpa);
    printf("log page maximum: %lu\n", dbs->dbslpm);
    printf("log page threshold: %lu\n", dbs->dbslpt);
    printf("last page written: %lu\n", dbs->dbslpw);
    printf("last temporary page accessed: %lu\n", dbs->dbsltp);
    printf("last temporary page written: %lu\n", dbs->dbsltw);
    printf("use count: %u\n",  dbs->dbsuse);
    printf("number of active transactions: %u\n",  dbs->dbsnat);
    printf("number of transactions: %u\n",  dbs->dbsntr);
  }
}

/* report process information */
rptPrc(msh, prc)
mshdef* msh;
prcdef *prc;
{
  int i;

  printf("\nPROCESSES:\n\n");
  printf("\nCurOutLen InBufLen InLen OutBufLen OutLen PNum Status\n");
  for (i = 0; i < msh->mshnen; prc++, i++)
  {
     printf("%9u %8u %5u %9u %6u %4u %s\n",
	 prc->prccol,			/* current output message length */
	 prc->prcibl,			/* input message buffer length */
	 prc->prcinl,			/* input length */
	 prc->prcobl,			/* output message buffer length */
	 prc->prcoul,			/* output length */
	 prc->prcpnm,			/* process number */
         prc->prcact ? "ACTIVE" : "IDLE"); /* active flag */
  }
}

/* report configuration information */
rptCfg(msh, cfg)
mshdef* msh;
cfgdef *cfg;
{
  printf("\nCONFIGURATION:\n\n");
  printf("Work space allocation size: %lu\n", cfg->cfgwsa);
  printf("Work space limit: %lu\n", cfg->cfgwsl);
  printf("number of locks: %u\n", cfg->cfgnlk);
  printf("number of pages: %u\n", cfg->cfgnpg);
  printf("connect count: %u\n", cfg->cfgcnc);
  printf("servername: %s\n", cfg->cfgsvn);
}

/* report statistical information */
rptStt(msh, stt)
mshdef* msh;
sttdef *stt;
{
  printf("\nSTATISTICS:\n\n");
  printf("physical reads: %lu\n", stt->sttspr);
  printf("physical writes: %lu\n", stt->sttspw);
  printf("virtual reads: %lu\n", stt->sttsvr);
  printf("virtual writes: %lu\n", stt->sttsvw);
  printf("number of process switches: %lu\n", stt->sttnps);
}

doerror(format, a1)
char *format;
char a1;
{
  printf("%s: ", Command);
  vprintf(format, &a1);
  exit(1);
}

doprint(format, a1)
char *format;
char a1;
{
  printf("%s: ", Command);
  vprintf(format, &a1);
}

/*
  Display help text
*/
void help(void)
{
  char **text;				/* -> list of help messages */
  char *s;				/* -> help message */

  text = Helptext;			/* -> list of help messages */
  while ((s = *text++) != 0)		/* process every help message */
  {
    while (*s)				/* process every character in msg */
    {
      if (*s == '%')			/* is special help character? */
      {
	if(s[1] == 'C')			/* is char to print program name? */
	{
	  printf("%s", Command);	/* print program name instead */
	  s++;
	}
      }
      else
      {
	putchar(*s);			/* print character in message */
      }
      s++;				/* advance to next character */
    }
    putchar('\n');			/* append line feed */
  }
  exit(0);				/* exit from program */
}
